@@ -12,15 +12,20 @@ namespace vcsn
 
   format::format(const std::string& f)
   {
-    static const auto map = std::map<std::string, format_t>
+    static const auto map = getarg<format_t>
       {
-        {"auto",    deflt},
-        {"default", deflt},
-        {"latex",   latex},
-        {"raw",     raw},
-        {"text",    text},
+        "format",
+        {
+          {"auto",       "default"},
+          {"default",    deflt},
+          {"latex",      latex},
+          {"raw",        raw},
+          {"sname",      sname},
+          {"text",       text},
+          {"utf8",       utf8},
+        }
       };
-    format_ = getargs("format", map, f);
+    format_ = map[f];
   }
 
   std::string to_string(format f)
@@ -31,8 +36,12 @@ namespace vcsn
         return "latex";
       case format::raw:
         return "raw";
+      case format::sname:
+        return "sname";
       case format::text:
         return "text";
+      case format::utf8:
+        return "utf8";
       }
     BUILTIN_UNREACHABLE();
   }
