@@ -3,14 +3,16 @@
 ## ------------ ##
 
 from vcsn_cxx import polynomial
-from vcsn import _is_equal, _right_mult
+from vcsn.tools import _is_equal, _right_mult
 
-polynomial.__eq__ = _is_equal
 polynomial.__add__ = polynomial.sum
+polynomial.__and__ = polynomial.conjunction
+polynomial.__eq__ = _is_equal
 polynomial.__mul__ = _right_mult
 polynomial.__repr__ = lambda self: self.format('text')
 polynomial._repr_latex_ = lambda self: '$' + self.format('latex') + '$'
 
+
 def _polynomial_pow(p, k):
     # FIXME: cannot generate the unit polynomial with the current API.
     assert 0 < k
