@@ -3,7 +3,9 @@
 ## -------- ##
 
 from vcsn_cxx import weight
-from vcsn import automaton, _is_equal, expression
+from vcsn import automaton, expression
+from vcsn.tools import _is_equal
+
 
 def _weight_pow(self, exp):
     if isinstance(exp, tuple):
@@ -11,12 +13,14 @@ def _weight_pow(self, exp):
     else:
         return self.multiply(exp)
 
+
 def _weight_mul(self, rhs):
     '''Translate `weight * expression` to `expression.left_mult(weight)`.'''
     if isinstance(rhs, automaton) or isinstance(rhs, expression):
         return rhs.left_mult(self)
     elif isinstance(rhs, int):
-        raise RuntimeError("cannot multiply a weight by and int: need two weights")
+        raise RuntimeError(
+            "cannot multiply a weight by and int: need two weights")
     else:
         return self.multiply(rhs)
 
