@@ -16,6 +16,7 @@
 #endif
 
 #include <boost/python.hpp>
+#include <boost/python/args.hpp>
 #include <vcsn/core/rat/identities.hh>
 #include <vcsn/dyn/algos.hh>
 #include <vcsn/dyn/context.hh> // vname
@@ -74,10 +75,21 @@ struct python_optional
   }
 };
 
+/// Convert to identities.
+auto identities(const std::string& s)
+  -> vcsn::rat::identities
+{
+  std::istringstream is{s};
+  vcsn::rat::identities res;
+  is >> res;
+  vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
+  return res;
+}
+
 /// Convert a Python list to a C++ vector.
 template <typename T>
-std::vector<T>
-make_vector(const boost::python::list& list)
+auto make_vector(const boost::python::list& list)
+  -> std::vector<T>
 {
   std::vector<T> res;
   for (int i = 0; i < boost::python::len(list); ++i)
@@ -85,14 +97,19 @@ make_vector(const boost::python::list& list)
   return res;
 }
 
-/// Convert to identities.
-vcsn::rat::identities identities(const std::string& s)
+/// Create an input stream from a file, or from a string.
+auto make_istream(const std::string& data = "",
+                  const std::string& filename = "")
+  -> std::shared_ptr<std::istream>
 {
-  std::istringstream is{s};
-  vcsn::rat::identities res;
-  is >> res;
-  vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
-  return res;
+  vcsn::require(!data.empty() || !filename.empty(),
+                "cannot provide both data and filename");
+  if (!data.empty())
+    return std::make_shared<std::istringstream>(data);
+  else if (!filename.empty())
+    return vcsn::open_input_file(filename);
+  else
+    vcsn::raise("must provide either data or filename");
 }
 
 
@@ -131,7 +148,9 @@ struct context
 
   automaton cerny(unsigned n) const;
 
-  automaton cotrie(const std::string& filename) const;
+  automaton cotrie(const std::string& data = "",
+                   const std::string& format = "default",
+                   const std::string& filename = "") const;
 
   automaton de_bruijn(unsigned n) const;
 
@@ -155,6 +174,8 @@ struct context
 
   automaton levenshtein() const;
 
+  context project(unsigned tape) const;
+
   automaton random(unsigned num_states, float density = 0.1,
                    unsigned num_initial = 1, unsigned num_final = 1,
                    float loop_chance = 0.0) const;
@@ -162,7 +183,9 @@ struct context
 
   expression series(const std::string& s) const;
 
-  automaton trie(const std::string& filename) const;
+  automaton trie(const std::string& data = "",
+                 const std::string& format = "default",
+                 const std::string& filename = "") const;
 
   automaton u(unsigned num_states) const;
 
@@ -190,15 +213,7 @@ struct automaton
             const std::string& filename = "",
             bool strip = true)
   {
-    std::shared_ptr<std::istream> is;
-    vcsn::require(!data.empty() || !filename.empty(),
-                  "cannot provide both data and filename");
-    if (!data.empty())
-      is = std::make_shared<std::istringstream>(data);
-    else if (!filename.empty())
-      is = vcsn::open_input_file(filename);
-    else
-      vcsn::raise("must provide either data or filename");
+    auto is = make_istream(data, filename);
     val_ = vcsn::dyn::read_automaton(*is, format, strip);
     vcsn::require(is->peek() == EOF, "unexpected trailing characters: ", *is);
   }
@@ -246,9 +261,9 @@ struct automaton
     return vcsn::dyn::component(val_, com_num);
   }
 
-  automaton compose(automaton& rhs)
+  automaton compose(automaton& rhs, bool lazy = false)
   {
-    return vcsn::dyn::compose(val_, rhs.val_);
+    return vcsn::dyn::compose(val_, rhs.val_, lazy);
   }
 
   automaton condense() const
@@ -256,9 +271,9 @@ struct automaton
     return vcsn::dyn::condense(val_);
   }
 
-  ::context context() const
+  automaton conjugate()
   {
-    return vcsn::dyn::context_of(val_);
+    return vcsn::dyn::conjugate(val_);
   }
 
   automaton conjunction(unsigned n) const
@@ -278,6 +293,11 @@ struct automaton
   using conjunction_variadic_t
     = auto (*)(const boost::python::list& auts, bool lazy) -> automaton;
 
+  ::context context() const
+  {
+    return vcsn::dyn::context_of(val_);
+  }
+
   automaton costandard() const
   {
     return vcsn::dyn::costandard(val_);
@@ -332,6 +352,11 @@ struct automaton
     return vcsn::dyn::has_bounded_lag(val_);
   }
 
+  bool has_lightening_cycle() const
+  {
+    return vcsn::dyn::has_lightening_cycle(val_);
+  }
+
   bool has_twins_property() const
   {
     return vcsn::dyn::has_twins_property(val_);
@@ -342,9 +367,9 @@ struct automaton
     return vcsn::dyn::infiltration(automata_(auts));
   }
 
-  automaton insplit() const
+  automaton insplit(bool lazy = false) const
   {
-    return vcsn::dyn::insplit(val_);
+    return vcsn::dyn::insplit(val_, lazy);
   }
 
   bool is_accessible() const
@@ -474,7 +499,8 @@ struct automaton
     return vcsn::dyn::is_valid(val_);
   }
 
-  automaton left_mult(const weight& w) const;
+  automaton left_mult(const weight& w,
+                      const std::string& algo = "auto") const;
 
   automaton letterize() const
   {
@@ -492,34 +518,50 @@ struct automaton
     = auto (automaton::*)(const boost::python::list& tapes,
                           const std::string& ids) const -> automaton;
 
-  polynomial lightest(boost::optional<unsigned> num) const;
+  polynomial lightest(unsigned num, const std::string& algo = "auto") const;
+
+  automaton lightest_automaton(const std::string& algo = "auto") const
+  {
+    return vcsn::dyn::lightest_automaton(val_, algo);
+  }
 
   automaton minimize(const std::string& algo = "auto") const
   {
     return vcsn::dyn::minimize(val_, algo);
   }
 
-  automaton multiply(const automaton& rhs) const
+  automaton multiply(const automaton& rhs,
+                     const std::string& algo = "auto") const
   {
-    return vcsn::dyn::multiply(val_, rhs.val_);
+    return vcsn::dyn::multiply(val_, rhs.val_, algo);
   }
   /// The type of the previous function.
   using multiply_t =
-    auto (automaton::*)(const automaton&) const -> automaton;
+    auto
+    (automaton::*)(const automaton&, const std::string& algo) const
+      -> automaton;
 
-  automaton multiply(int min, int max) const
+  automaton multiply(int min, int max, const std::string& algo = "auto") const
   {
-    return vcsn::dyn::multiply(val_, min, max);
+    return vcsn::dyn::multiply(val_, min, max, algo);
   }
   /// The type of the previous function.
   using multiply_repeated_t =
-    auto (automaton::*)(int min, int max) const -> automaton;
+    auto
+    (automaton::*)(int min, int max, const std::string& algo) const
+      -> automaton;
 
-  automaton multiply(int min) const
+  automaton multiply(int min, const std::string& algo = "auto") const
   {
-    return multiply(min, min);
+    return multiply(min, min, algo);
   }
 
+  /// The type of the previous function.
+  using multiply_min_t =
+    auto
+    (automaton::*)(int min, const std::string& algo) const
+      -> automaton;
+
   automaton normalize() const
   {
     return vcsn::dyn::normalize(val_);
@@ -551,7 +593,7 @@ struct automaton
   }
 
   automaton proper(bool prune = true, bool backward = true,
-                   const std::string algo = "auto") const
+                   const std::string& algo = "auto") const
   {
     return vcsn::dyn::proper(val_,
                              backward
@@ -575,7 +617,8 @@ struct automaton
     return vcsn::dyn::reduce(val_);
   }
 
-  automaton right_mult(const weight& w) const;
+  automaton right_mult(const weight& w,
+                       const std::string& algo = "auto") const;
 
   automaton scc(const std::string& algo = "auto") const
   {
@@ -600,9 +643,9 @@ struct automaton
     return vcsn::dyn::standard(val_);
   }
 
-  automaton star() const
+  automaton star(const std::string& algo = "auto") const
   {
-    return vcsn::dyn::star(val_);
+    return vcsn::dyn::star(val_, algo);
   }
 
   automaton strip() const
@@ -620,9 +663,9 @@ struct automaton
     return vcsn::dyn::subword(val_);
   }
 
-  automaton sum(const automaton& rhs) const
+  automaton sum(const automaton& rhs, const std::string& algo = "auto") const
   {
-    return vcsn::dyn::sum(val_, rhs.val_);
+    return vcsn::dyn::sum(val_, rhs.val_, algo);
   }
 
   automaton synchronize() const
@@ -650,11 +693,6 @@ struct automaton
     return vcsn::dyn::type(val_);
   }
 
-  automaton union_a(const automaton& rhs) const
-  {
-    return vcsn::dyn::union_a(val_, rhs.val_);
-  }
-
   automaton universal() const
   {
     return vcsn::dyn::universal(val_);
@@ -690,6 +728,16 @@ struct expansion
     vcsn::raise("not implemented");
   }
 
+  expansion complement() const
+  {
+    return vcsn::dyn::complement(val_);
+  }
+
+  ::context context() const
+  {
+    return vcsn::dyn::context_of(val_);
+  }
+
   std::string format(const std::string& format = "text") const
   {
     std::ostringstream os;
@@ -697,6 +745,30 @@ struct expansion
     return os.str();
   }
 
+  expansion sum(const expansion& rhs) const
+  {
+    return vcsn::dyn::sum(val_, rhs.val_);
+  }
+
+  expansion left_mult(const weight& w) const;
+
+  expansion right_mult(const weight& w) const;
+
+  static expansion tuple_(const boost::python::list& es)
+  {
+    return vcsn::dyn::tuple(expansions_(es));
+  }
+
+  /// Convert to a vector of dyn:: expansions.
+  using expansions_t = std::vector<vcsn::dyn::expansion>;
+  static expansions_t expansions_(const boost::python::list& es)
+  {
+    auto res = expansions_t{};
+    for (int i = 0; i < boost::python::len(es); ++i)
+      res.emplace_back(boost::python::extract<expansion>(es[i])().val_);
+    return res;
+  }
+
   vcsn::dyn::expansion val_;
 };
 
@@ -729,6 +801,11 @@ struct label
     return vcsn::dyn::multiply(val_, rhs.val_);
   }
 
+  label project(unsigned tape) const
+  {
+    return vcsn::dyn::project(val_, tape);
+  }
+
   vcsn::dyn::label val_;
 };
 
@@ -749,6 +826,11 @@ struct polynomial
     vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
   }
 
+  polynomial conjunction(const polynomial& rhs) const
+  {
+    return vcsn::dyn::conjunction(val_, rhs.val_);
+  }
+
   automaton cotrie() const
   {
     return vcsn::dyn::cotrie(val_);
@@ -776,6 +858,11 @@ struct polynomial
     return vcsn::dyn::multiply(val_, rhs.val_);
   }
 
+  polynomial project(unsigned tape)
+  {
+    return vcsn::dyn::project(val_, tape);
+  }
+
   polynomial split() const
   {
     return vcsn::dyn::split(val_);
@@ -1112,19 +1199,21 @@ bool automaton::is_synchronized_by(const label& word) const
   return vcsn::dyn::is_synchronized_by(val_, word.val_);
 }
 
-automaton automaton::left_mult(const weight& w) const
+automaton automaton::left_mult(const weight& w,
+                               const std::string& algo) const
 {
-  return vcsn::dyn::left_mult(w.val_, val_);
+  return vcsn::dyn::left_mult(w.val_, val_, algo);
 }
 
-polynomial automaton::lightest(boost::optional<unsigned> num) const
+polynomial automaton::lightest(unsigned num, const std::string& algo) const
 {
-  return vcsn::dyn::lightest(val_, num);
+  return vcsn::dyn::lightest(val_, num, algo);
 }
 
-automaton automaton::right_mult(const weight& w) const
+automaton automaton::right_mult(const weight& w,
+                                const std::string& algo) const
 {
-  return vcsn::dyn::right_mult(val_, w.val_);
+  return vcsn::dyn::right_mult(val_, w.val_, algo);
 }
 
 polynomial automaton::shortest(boost::optional<unsigned> num,
@@ -1158,10 +1247,12 @@ automaton context::cerny(unsigned n) const
   return vcsn::dyn::cerny(val_, n);
 }
 
-automaton context::cotrie(const std::string& filename) const
+automaton context::cotrie(const std::string& data,
+                          const std::string& format,
+                          const std::string& filename) const
 {
-  auto is = vcsn::open_input_file(filename);
-  auto res = vcsn::dyn::cotrie(val_, *is);
+  auto is = make_istream(data, filename);
+  auto res = vcsn::dyn::cotrie(val_, *is, format);
   vcsn::require(is->peek() == EOF, "unexpected trailing characters: ", *is);
   return res;
 }
@@ -1192,6 +1283,10 @@ automaton context::levenshtein() const
   return vcsn::dyn::levenshtein(val_);
 }
 
+context context::project(unsigned tape) const
+{
+  return vcsn::dyn::project(val_, tape);
+}
 
 automaton context::random(unsigned num_states, float density,
                           unsigned num_initial, unsigned num_final,
@@ -1207,10 +1302,12 @@ automaton context::random_deterministic(unsigned num_states) const
   return vcsn::dyn::random_automaton_deterministic(val_, num_states);
 }
 
-automaton context::trie(const std::string& filename) const
+automaton context::trie(const std::string& data,
+                        const std::string& format,
+                        const std::string& filename) const
 {
-  auto is = vcsn::open_input_file(filename);
-  auto res = vcsn::dyn::trie(val_, *is);
+  auto is = make_istream(data, filename);
+  auto res = vcsn::dyn::trie(val_, *is, format);
   vcsn::require(is->peek() == EOF, "unexpected trailing characters: ", *is);
   return res;
 }
@@ -1225,7 +1322,20 @@ label context::word(const std::string& s) const
   return label(context(vcsn::dyn::make_word_context(val_)), s);
 }
 
-
+/*----------------------------.
+| expansion implementation.   |
+`----------------------------*/
+
+expansion expansion::left_mult(const weight& w) const
+{
+  return vcsn::dyn::left_mult(w.val_, val_);
+}
+
+expansion expansion::right_mult(const weight& w) const
+{
+  return vcsn::dyn::right_mult(val_, w.val_);
+}
+
 /*-----------------------------.
 | expression implementation.   |
 `-----------------------------*/
@@ -1254,6 +1364,8 @@ BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(lift,
                                        lift, 0, 2);
 BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(multiply_repeated,
                                        multiply, 1, 2);
+BOOST_PYTHON_MEMBER_FUNCTION_OVERLOADS(multiply_repeated_automaton,
+                                       multiply, 1, 3);
 
 BOOST_PYTHON_MODULE(vcsn_cxx)
 {
@@ -1282,7 +1394,7 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("complement", &automaton::complement)
     .def("complete", &automaton::complete)
     .def("component", &automaton::component)
-    .def("compose", &automaton::compose)
+    .def("compose", &automaton::compose, (arg("lazy") = false))
     .def("condense", &automaton::condense)
     .def("conjunction",
          static_cast<automaton::conjunction_repeated_t>(&automaton::conjunction))
@@ -1290,6 +1402,7 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
          static_cast<automaton::conjunction_variadic_t>(&automaton::conjunction),
          (arg("lazy") = false))
         .staticmethod("conjunction")
+    .def("conjugate", &automaton::conjugate)
     .def("context", &automaton::context)
     .def("costandard", &automaton::costandard)
     .def("delay_automaton", &automaton::delay_automaton)
@@ -1301,9 +1414,10 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("filter", &automaton::filter)
     .def("_format", &automaton::format)
     .def("has_bounded_lag", &automaton::has_bounded_lag)
+    .def("has_lightening_cycle", &automaton::has_lightening_cycle)
     .def("has_twins_property", &automaton::has_twins_property)
     .def("_infiltration", &automaton::infiltration_).staticmethod("_infiltration")
-    .def("insplit", &automaton::insplit)
+    .def("insplit", &automaton::insplit, (arg("lazy") = false))
     .def("is_accessible", &automaton::is_accessible)
     .def("is_ambiguous", &automaton::is_ambiguous)
     .def("is_coaccessible", &automaton::is_coaccessible)
@@ -1330,16 +1444,23 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("is_trim", &automaton::is_trim)
     .def("is_useless", &automaton::is_useless)
     .def("is_valid", &automaton::is_valid)
-    .def("left_mult", &automaton::left_mult)
+    .def("left_mult", &automaton::left_mult,
+         (arg("weight"), arg("algo") = "auto"))
     .def("letterize", &automaton::letterize)
     .def("_lift", &automaton::lift)
     .def("lightest", &automaton::lightest,
-         (arg("num") = boost::optional<unsigned>()))
+         (arg("num") = 1U, arg("algo") = "auto"))
+    .def("lightest_automaton",
+         &automaton::lightest_automaton, (arg("algo") = "auto"))
     .def("minimize", &automaton::minimize, (arg("algo") = "auto"))
-    .def("multiply", static_cast<automaton::multiply_t>(&automaton::multiply))
+    .def("multiply", static_cast<automaton::multiply_t>(&automaton::multiply),
+         (arg("algo") = "auto"))
     .def("multiply",
          static_cast<automaton::multiply_repeated_t>(&automaton::multiply),
-         multiply_repeated())
+         multiply_repeated_automaton(bp::args("min", "max", "algo")))
+    .def("multiply",
+         static_cast<automaton::multiply_min_t>(&automaton::multiply),
+         multiply_repeated(bp::args("min", "algo")))
     .def("normalize", &automaton::normalize)
     .def("num_components", &automaton::num_components)
     .def("pair", &automaton::pair, (arg("keep_initials") = false))
@@ -1353,7 +1474,8 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("expression", &automaton::to_expression,
          (arg("identities") = "default", arg("algo") = "auto"))
     .def("reduce", &automaton::reduce)
-    .def("right_mult", &automaton::right_mult)
+    .def("right_mult", &automaton::right_mult,
+         (arg("weight"), arg("algo") = "auto"))
     .def("scc", &automaton::scc, (arg("algo") = "auto"))
     .def("shortest", &automaton::shortest,
          (arg("num") = boost::optional<unsigned>(),
@@ -1361,18 +1483,17 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("_shuffle", &automaton::shuffle_).staticmethod("_shuffle")
     .def("sort", &automaton::sort)
     .def("standard", &automaton::standard)
-    .def("star", &automaton::star)
+    .def("star", &automaton::star, (arg("algo") = "auto"))
     .def("strip", &automaton::strip)
     .def("suffix", &automaton::suffix)
     .def("subword", &automaton::subword)
-    .def("sum", &automaton::sum)
+    .def("sum", &automaton::sum, (arg("algo") = "auto"))
     .def("synchronize", &automaton::synchronize)
     .def("synchronizing_word",
          &automaton::synchronizing_word, (arg("algo") = "greedy"))
     .def("transpose", &automaton::transpose)
     .def("trim", &automaton::trim)
     .def("type", &automaton::type)
-    .def("union", &automaton::union_a)
     .def("universal", &automaton::universal)
     .def("weight_series", &automaton::weight_series)
     ;
@@ -1380,7 +1501,9 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
   bp::class_<context>("context", bp::no_init)
     .def(bp::init<const std::string&>())
     .def("cerny", &context::cerny)
-    .def("cotrie", &context::cotrie)
+    .def("cotrie", &context::cotrie,
+         (arg("data") = "", arg("format") = "default",
+          arg("filename") = ""))
     .def("de_bruijn", &context::de_bruijn)
     .def("divkbaseb", &context::divkbaseb)
     .def("double_ring", &context::double_ring)
@@ -1388,25 +1511,35 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("join", &context::join)
     .def("ladybird", &context::ladybird)
     .def("levenshtein", &context::levenshtein)
+    .def("project", &context::project)
     .def("random", &context::random,
          (arg("num_states"), arg("density") = 0.1,
-          arg("num_initial") = 1, arg("num_final") = 1),
-          arg("loop_chance") = 0)
+          arg("num_initial") = 1, arg("num_final") = 1,
+          arg("loop_chance") = 0))
     .def("random_deterministic", &context::random_deterministic)
     .def("series", &context::series)
-    .def("trie", &context::trie)
+    .def("trie", &context::trie,
+         (arg("data") = "", arg("format") = "default",
+          arg("filename") = ""))
     .def("u", &context::u)
     .def("word", &context::word)
    ;
 
   bp::class_<expansion>("expansion", bp::no_init)
     .def(bp::init<const std::string&>())
+    .def("complement", &expansion::complement)
+    .def("context", &expansion::context)
     .def("format", &expansion::format)
+    .def("left_mult", &expansion::left_mult)
+    .def("right_mult", &expansion::right_mult)
+    .def("sum", &expansion::sum)
+    .def("_tuple", &expansion::tuple_).staticmethod("_tuple")
    ;
 
   bp::class_<expression>("expression", bp::no_init)
     .def(bp::init<const context&, const std::string&, const std::string&>
          ((arg("context"), arg("data"), arg("identities") = "default")))
+    .def("automaton", &expression::to_automaton, (arg("algo") = "auto"))
     .def("complement", &expression::complement)
     .def("conjunction", &expression::conjunction)
     .def("constant_term", &expression::constant_term)
@@ -1424,6 +1557,7 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("is_equivalent", &expression::is_equivalent)
     .def("is_series", &expression::is_series)
     .def("is_valid", &expression::is_valid)
+    .def("ldiv", &expression::ldiv)
     .def("left_mult", &expression::left_mult)
     .def("less_than", &expression::less_than)
     .def("lift", &expression::lift)
@@ -1431,6 +1565,7 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("multiply",
          static_cast<expression::multiply_repeated_t>(&expression::multiply),
          multiply_repeated())
+    .def("rdiv", &expression::rdiv)
     .def("right_mult", &expression::right_mult)
     .def("series", &expression::as_series, (arg("context") = context()))
     .def("shuffle", &expression::shuffle)
@@ -1440,7 +1575,6 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def("star_normal_form", &expression::star_normal_form)
     .def("sum", &expression::sum)
     .def("thompson", &expression::thompson)
-    .def("automaton", &expression::to_automaton, (arg("algo") = "auto"))
     .def("transpose", &expression::transpose)
     .def("transposition", &expression::transposition)
     .def("_tuple", &expression::tuple_).staticmethod("_tuple")
@@ -1451,15 +1585,18 @@ BOOST_PYTHON_MODULE(vcsn_cxx)
     .def(bp::init<const context&, const std::string&>())
     .def("format", &label::format)
     .def("multiply", &label::multiply)
+    .def("project", &label::project)
    ;
 
   bp::class_<polynomial>("polynomial", bp::no_init)
     .def(bp::init<const context&, const std::string&>())
+    .def("conjunction", &polynomial::conjunction)
     .def("cotrie", &polynomial::cotrie)
     .def("format", &polynomial::format)
     .def("ldiv", &polynomial::ldiv)
     .def("lgcd", &polynomial::lgcd)
     .def("multiply", &polynomial::multiply)
+    .def("project", &polynomial::project)
     .def("split", &polynomial::split)
     .def("sum", &polynomial::sum)
     .def("trie", &polynomial::trie)
