@@ -228,7 +228,7 @@ a2 = vcsn.context('lal_char(ab), seriesset<lal_char(xy), q>') \
          .expression('<x>a<y>b').standard()
 
 def check_enumerate(exp, aut):
-    CHECK_EQ(exp, aut.strip().shortest(len = 4))
+    CHECK_EQ(exp, aut.strip().shortest(len=4))
 
 check_enumerate('<uxvy>ab', a1 & a2)
 check_enumerate('\z', a1.transpose() & a2)
@@ -294,6 +294,35 @@ q = vcsn.context('lal_char(c), q').expression('<1/3>c*')
 r = vcsn.context('lal_char(d), r').expression('<.4>d*')
 CHECK_EQ('<u>a*&<<2>\e>b*&<<0.333333>\e>c*&<<0.4>\e>d*', qr & z & q & r)
 
+## ------------------------- ##
+## polynomial & polynomial.  ##
+## ------------------------- ##
+
+poly = vcsn.context('lal_char(abc), q').polynomial
+a = poly('<2>a+<3>b')
+b = poly('<2>b+<3>c')
+c = poly('<3>c')
+CHECK_EQ('<4>a + <9>b', a & a)
+CHECK_EQ('<6>b', a & b)
+CHECK_EQ('<6>b', b & a)
+CHECK_EQ('\z', a & c)
+CHECK_EQ('<9>c', b & c)
+
+poly = vcsn.context('lan_char, q').polynomial
+a = poly('<2>\e')
+b = poly('<2>a+<3>\e')
+c = poly('<2>a+<3>c')
+CHECK_EQ('<4>\e', a & a)
+CHECK_EQ('\z', a & c)
+CHECK_EQ('<6>\e', a & b)
+CHECK_EQ('<4>a', b & c)
+
+poly = vcsn.context('law_char, q').polynomial
+a = poly('<1>aa')
+b = poly('<2>ab')
+CHECK_EQ('\z', a & b)
+CHECK_EQ('aa', a & a)
+
 ## ----------------- ##
 ## nullable labels.  ##
 ## ----------------- ##
@@ -312,32 +341,32 @@ res = r'''digraph
   }
   {
     node [shape = circle, style = rounded, width = 0.5]
-    0 [label = "6, 6", shape = box]
-    1 [label = "0, 6", shape = box]
-    2 [label = "7, 6", shape = box]
-    3 [label = "2, 6", shape = box, color = DimGray]
-    4 [label = "4, 6", shape = box]
-    5 [label = "7, 0", shape = box, color = DimGray]
-    6 [label = "7, 7", shape = box]
-    7 [label = "2, 0", shape = box, color = DimGray]
-    8 [label = "2, 7", shape = box, color = DimGray]
-    9 [label = "4, 0", shape = box]
-    10 [label = "4, 7", shape = box, color = DimGray]
-    11 [label = "7, 2", shape = box, color = DimGray]
-    12 [label = "7, 4", shape = box, color = DimGray]
-    13 [label = "2, 2", shape = box, color = DimGray]
-    14 [label = "2, 4", shape = box, color = DimGray]
-    15 [label = "4, 2", shape = box]
-    16 [label = "4, 4", shape = box, color = DimGray]
-    17 [label = "5, 3", shape = box]
-    18 [label = "1, 3", shape = box]
-    19 [label = "7, 3", shape = box]
-    20 [label = "0, 3", shape = box]
-    21 [label = "7, 1", shape = box]
-    22 [label = "2, 3", shape = box, color = DimGray]
-    23 [label = "4, 3", shape = box]
-    24 [label = "2, 1", shape = box, color = DimGray]
-    25 [label = "4, 1", shape = box]
+    0 [label = "6, (6, !\\e)", shape = box]
+    1 [label = "0, (6, !\\e)", shape = box]
+    2 [label = "7, (6, !\\e)", shape = box]
+    3 [label = "2, (6, !\\e)", shape = box, color = DimGray]
+    4 [label = "4, (6, !\\e)", shape = box]
+    5 [label = "7, (0, \\e)", shape = box, color = DimGray]
+    6 [label = "7, (7, \\e)", shape = box]
+    7 [label = "2, (0, \\e)", shape = box, color = DimGray]
+    8 [label = "2, (7, \\e)", shape = box, color = DimGray]
+    9 [label = "4, (0, \\e)", shape = box]
+    10 [label = "4, (7, \\e)", shape = box, color = DimGray]
+    11 [label = "7, (2, \\e)", shape = box, color = DimGray]
+    12 [label = "7, (4, \\e)", shape = box, color = DimGray]
+    13 [label = "2, (2, \\e)", shape = box, color = DimGray]
+    14 [label = "2, (4, \\e)", shape = box, color = DimGray]
+    15 [label = "4, (2, \\e)", shape = box]
+    16 [label = "4, (4, \\e)", shape = box, color = DimGray]
+    17 [label = "5, (3, !\\e)", shape = box]
+    18 [label = "1, (3, !\\e)", shape = box]
+    19 [label = "7, (3, !\\e)", shape = box]
+    20 [label = "0, (3, !\\e)", shape = box]
+    21 [label = "7, (1, \\e)", shape = box]
+    22 [label = "2, (3, !\\e)", shape = box, color = DimGray]
+    23 [label = "4, (3, !\\e)", shape = box]
+    24 [label = "2, (1, \\e)", shape = box, color = DimGray]
+    25 [label = "4, (1, \\e)", shape = box]
   }
   I0 -> 0
   0 -> 1 [label = "\\e"]
@@ -390,98 +419,98 @@ res = r'''digraph
   }
   {
     node [shape = circle, style = rounded, width = 0.5]
-    0 [label = "6, 6, 8", shape = box]
-    1 [label = "0, 6, 8", shape = box]
-    2 [label = "7, 6, 8", shape = box]
-    3 [label = "2, 6, 8", shape = box, color = DimGray]
-    4 [label = "4, 6, 8", shape = box]
-    5 [label = "7, 0, 8", shape = box, color = DimGray]
-    6 [label = "7, 7, 8", shape = box]
-    7 [label = "2, 0, 8", shape = box, color = DimGray]
-    8 [label = "2, 7, 8", shape = box, color = DimGray]
-    9 [label = "4, 0, 8", shape = box]
-    10 [label = "4, 7, 8", shape = box, color = DimGray]
-    11 [label = "7, 2, 8", shape = box, color = DimGray]
-    12 [label = "7, 4, 8", shape = box, color = DimGray]
-    13 [label = "7, 7, 0", shape = box, color = DimGray]
-    14 [label = "7, 7, 9", shape = box]
-    15 [label = "2, 2, 8", shape = box, color = DimGray]
-    16 [label = "2, 4, 8", shape = box, color = DimGray]
-    17 [label = "2, 7, 0", shape = box, color = DimGray]
-    18 [label = "2, 7, 9", shape = box, color = DimGray]
-    19 [label = "4, 2, 8", shape = box]
-    20 [label = "4, 4, 8", shape = box, color = DimGray]
-    21 [label = "4, 7, 0", shape = box, color = DimGray]
-    22 [label = "4, 7, 9", shape = box, color = DimGray]
-    23 [label = "7, 2, 0", shape = box, color = DimGray]
-    24 [label = "7, 2, 9", shape = box, color = DimGray]
-    25 [label = "7, 4, 0", shape = box, color = DimGray]
-    26 [label = "7, 4, 9", shape = box, color = DimGray]
-    27 [label = "7, 7, 2", shape = box, color = DimGray]
-    28 [label = "7, 7, 4", shape = box, color = DimGray]
-    29 [label = "7, 7, 6", shape = box, color = DimGray]
-    30 [label = "2, 2, 0", shape = box, color = DimGray]
-    31 [label = "2, 2, 9", shape = box, color = DimGray]
-    32 [label = "2, 4, 0", shape = box, color = DimGray]
-    33 [label = "2, 4, 9", shape = box, color = DimGray]
-    34 [label = "2, 7, 2", shape = box, color = DimGray]
-    35 [label = "2, 7, 4", shape = box, color = DimGray]
-    36 [label = "2, 7, 6", shape = box, color = DimGray]
-    37 [label = "4, 2, 0", shape = box]
-    38 [label = "4, 2, 9", shape = box, color = DimGray]
-    39 [label = "4, 4, 0", shape = box, color = DimGray]
-    40 [label = "4, 4, 9", shape = box, color = DimGray]
-    41 [label = "4, 7, 2", shape = box, color = DimGray]
-    42 [label = "4, 7, 4", shape = box, color = DimGray]
-    43 [label = "4, 7, 6", shape = box, color = DimGray]
-    44 [label = "7, 2, 2", shape = box, color = DimGray]
-    45 [label = "7, 2, 4", shape = box, color = DimGray]
-    46 [label = "7, 2, 6", shape = box, color = DimGray]
-    47 [label = "7, 4, 2", shape = box, color = DimGray]
-    48 [label = "7, 4, 4", shape = box, color = DimGray]
-    49 [label = "7, 4, 6", shape = box, color = DimGray]
-    50 [label = "2, 2, 2", shape = box, color = DimGray]
-    51 [label = "2, 2, 4", shape = box, color = DimGray]
-    52 [label = "2, 2, 6", shape = box, color = DimGray]
-    53 [label = "2, 4, 2", shape = box, color = DimGray]
-    54 [label = "2, 4, 4", shape = box, color = DimGray]
-    55 [label = "2, 4, 6", shape = box, color = DimGray]
-    56 [label = "4, 2, 2", shape = box]
-    57 [label = "4, 2, 4", shape = box, color = DimGray]
-    58 [label = "4, 2, 6", shape = box, color = DimGray]
-    59 [label = "4, 4, 2", shape = box, color = DimGray]
-    60 [label = "4, 4, 4", shape = box, color = DimGray]
-    61 [label = "4, 4, 6", shape = box, color = DimGray]
-    62 [label = "5, 3, 3", shape = box]
-    63 [label = "1, 3, 3", shape = box]
-    64 [label = "7, 3, 3", shape = box]
-    65 [label = "0, 3, 3", shape = box]
-    66 [label = "7, 1, 3", shape = box]
-    67 [label = "2, 3, 3", shape = box, color = DimGray]
-    68 [label = "4, 3, 3", shape = box]
-    69 [label = "7, 7, 3", shape = box]
-    70 [label = "7, 0, 3", shape = box, color = DimGray]
-    71 [label = "2, 1, 3", shape = box, color = DimGray]
-    72 [label = "4, 1, 3", shape = box]
-    73 [label = "7, 7, 1", shape = box]
-    74 [label = "7, 2, 3", shape = box, color = DimGray]
-    75 [label = "7, 4, 3", shape = box, color = DimGray]
-    76 [label = "2, 7, 3", shape = box, color = DimGray]
-    77 [label = "2, 0, 3", shape = box, color = DimGray]
-    78 [label = "4, 7, 3", shape = box, color = DimGray]
-    79 [label = "4, 0, 3", shape = box]
-    80 [label = "7, 2, 1", shape = box, color = DimGray]
-    81 [label = "7, 4, 1", shape = box, color = DimGray]
-    82 [label = "2, 7, 1", shape = box, color = DimGray]
-    83 [label = "2, 2, 3", shape = box, color = DimGray]
-    84 [label = "2, 4, 3", shape = box, color = DimGray]
-    85 [label = "4, 7, 1", shape = box, color = DimGray]
-    86 [label = "4, 2, 3", shape = box]
-    87 [label = "4, 4, 3", shape = box, color = DimGray]
-    88 [label = "2, 2, 1", shape = box, color = DimGray]
-    89 [label = "2, 4, 1", shape = box, color = DimGray]
-    90 [label = "4, 2, 1", shape = box]
-    91 [label = "4, 4, 1", shape = box, color = DimGray]
+    0 [label = "6, (6, !\\e), (8, !\\e)", shape = box]
+    1 [label = "0, (6, !\\e), (8, !\\e)", shape = box]
+    2 [label = "7, (6, !\\e), (8, !\\e)", shape = box]
+    3 [label = "2, (6, !\\e), (8, !\\e)", shape = box, color = DimGray]
+    4 [label = "4, (6, !\\e), (8, !\\e)", shape = box]
+    5 [label = "7, (0, \\e), (8, !\\e)", shape = box, color = DimGray]
+    6 [label = "7, (7, \\e), (8, !\\e)", shape = box]
+    7 [label = "2, (0, \\e), (8, !\\e)", shape = box, color = DimGray]
+    8 [label = "2, (7, \\e), (8, !\\e)", shape = box, color = DimGray]
+    9 [label = "4, (0, \\e), (8, !\\e)", shape = box]
+    10 [label = "4, (7, \\e), (8, !\\e)", shape = box, color = DimGray]
+    11 [label = "7, (2, \\e), (8, !\\e)", shape = box, color = DimGray]
+    12 [label = "7, (4, \\e), (8, !\\e)", shape = box, color = DimGray]
+    13 [label = "7, (7, \\e), (0, \\e)", shape = box, color = DimGray]
+    14 [label = "7, (7, \\e), (9, \\e)", shape = box]
+    15 [label = "2, (2, \\e), (8, !\\e)", shape = box, color = DimGray]
+    16 [label = "2, (4, \\e), (8, !\\e)", shape = box, color = DimGray]
+    17 [label = "2, (7, \\e), (0, \\e)", shape = box, color = DimGray]
+    18 [label = "2, (7, \\e), (9, \\e)", shape = box, color = DimGray]
+    19 [label = "4, (2, \\e), (8, !\\e)", shape = box]
+    20 [label = "4, (4, \\e), (8, !\\e)", shape = box, color = DimGray]
+    21 [label = "4, (7, \\e), (0, \\e)", shape = box, color = DimGray]
+    22 [label = "4, (7, \\e), (9, \\e)", shape = box, color = DimGray]
+    23 [label = "7, (2, \\e), (0, \\e)", shape = box, color = DimGray]
+    24 [label = "7, (2, \\e), (9, \\e)", shape = box, color = DimGray]
+    25 [label = "7, (4, \\e), (0, \\e)", shape = box, color = DimGray]
+    26 [label = "7, (4, \\e), (9, \\e)", shape = box, color = DimGray]
+    27 [label = "7, (7, \\e), (2, \\e)", shape = box, color = DimGray]
+    28 [label = "7, (7, \\e), (4, \\e)", shape = box, color = DimGray]
+    29 [label = "7, (7, \\e), (6, \\e)", shape = box, color = DimGray]
+    30 [label = "2, (2, \\e), (0, \\e)", shape = box, color = DimGray]
+    31 [label = "2, (2, \\e), (9, \\e)", shape = box, color = DimGray]
+    32 [label = "2, (4, \\e), (0, \\e)", shape = box, color = DimGray]
+    33 [label = "2, (4, \\e), (9, \\e)", shape = box, color = DimGray]
+    34 [label = "2, (7, \\e), (2, \\e)", shape = box, color = DimGray]
+    35 [label = "2, (7, \\e), (4, \\e)", shape = box, color = DimGray]
+    36 [label = "2, (7, \\e), (6, \\e)", shape = box, color = DimGray]
+    37 [label = "4, (2, \\e), (0, \\e)", shape = box]
+    38 [label = "4, (2, \\e), (9, \\e)", shape = box, color = DimGray]
+    39 [label = "4, (4, \\e), (0, \\e)", shape = box, color = DimGray]
+    40 [label = "4, (4, \\e), (9, \\e)", shape = box, color = DimGray]
+    41 [label = "4, (7, \\e), (2, \\e)", shape = box, color = DimGray]
+    42 [label = "4, (7, \\e), (4, \\e)", shape = box, color = DimGray]
+    43 [label = "4, (7, \\e), (6, \\e)", shape = box, color = DimGray]
+    44 [label = "7, (2, \\e), (2, \\e)", shape = box, color = DimGray]
+    45 [label = "7, (2, \\e), (4, \\e)", shape = box, color = DimGray]
+    46 [label = "7, (2, \\e), (6, \\e)", shape = box, color = DimGray]
+    47 [label = "7, (4, \\e), (2, \\e)", shape = box, color = DimGray]
+    48 [label = "7, (4, \\e), (4, \\e)", shape = box, color = DimGray]
+    49 [label = "7, (4, \\e), (6, \\e)", shape = box, color = DimGray]
+    50 [label = "2, (2, \\e), (2, \\e)", shape = box, color = DimGray]
+    51 [label = "2, (2, \\e), (4, \\e)", shape = box, color = DimGray]
+    52 [label = "2, (2, \\e), (6, \\e)", shape = box, color = DimGray]
+    53 [label = "2, (4, \\e), (2, \\e)", shape = box, color = DimGray]
+    54 [label = "2, (4, \\e), (4, \\e)", shape = box, color = DimGray]
+    55 [label = "2, (4, \\e), (6, \\e)", shape = box, color = DimGray]
+    56 [label = "4, (2, \\e), (2, \\e)", shape = box]
+    57 [label = "4, (2, \\e), (4, \\e)", shape = box, color = DimGray]
+    58 [label = "4, (2, \\e), (6, \\e)", shape = box, color = DimGray]
+    59 [label = "4, (4, \\e), (2, \\e)", shape = box, color = DimGray]
+    60 [label = "4, (4, \\e), (4, \\e)", shape = box, color = DimGray]
+    61 [label = "4, (4, \\e), (6, \\e)", shape = box, color = DimGray]
+    62 [label = "5, (3, !\\e), (3, !\\e)", shape = box]
+    63 [label = "1, (3, !\\e), (3, !\\e)", shape = box]
+    64 [label = "7, (3, !\\e), (3, !\\e)", shape = box]
+    65 [label = "0, (3, !\\e), (3, !\\e)", shape = box]
+    66 [label = "7, (1, \\e), (3, !\\e)", shape = box]
+    67 [label = "2, (3, !\\e), (3, !\\e)", shape = box, color = DimGray]
+    68 [label = "4, (3, !\\e), (3, !\\e)", shape = box]
+    69 [label = "7, (7, \\e), (3, !\\e)", shape = box]
+    70 [label = "7, (0, \\e), (3, !\\e)", shape = box, color = DimGray]
+    71 [label = "2, (1, \\e), (3, !\\e)", shape = box, color = DimGray]
+    72 [label = "4, (1, \\e), (3, !\\e)", shape = box]
+    73 [label = "7, (7, \\e), (1, \\e)", shape = box]
+    74 [label = "7, (2, \\e), (3, !\\e)", shape = box, color = DimGray]
+    75 [label = "7, (4, \\e), (3, !\\e)", shape = box, color = DimGray]
+    76 [label = "2, (7, \\e), (3, !\\e)", shape = box, color = DimGray]
+    77 [label = "2, (0, \\e), (3, !\\e)", shape = box, color = DimGray]
+    78 [label = "4, (7, \\e), (3, !\\e)", shape = box, color = DimGray]
+    79 [label = "4, (0, \\e), (3, !\\e)", shape = box]
+    80 [label = "7, (2, \\e), (1, \\e)", shape = box, color = DimGray]
+    81 [label = "7, (4, \\e), (1, \\e)", shape = box, color = DimGray]
+    82 [label = "2, (7, \\e), (1, \\e)", shape = box, color = DimGray]
+    83 [label = "2, (2, \\e), (3, !\\e)", shape = box, color = DimGray]
+    84 [label = "2, (4, \\e), (3, !\\e)", shape = box, color = DimGray]
+    85 [label = "4, (7, \\e), (1, \\e)", shape = box, color = DimGray]
+    86 [label = "4, (2, \\e), (3, !\\e)", shape = box]
+    87 [label = "4, (4, \\e), (3, !\\e)", shape = box, color = DimGray]
+    88 [label = "2, (2, \\e), (1, \\e)", shape = box, color = DimGray]
+    89 [label = "2, (4, \\e), (1, \\e)", shape = box, color = DimGray]
+    90 [label = "4, (2, \\e), (1, \\e)", shape = box]
+    91 [label = "4, (4, \\e), (1, \\e)", shape = box, color = DimGray]
   }
   I0 -> 0
   0 -> 1 [label = "\\e"]
@@ -661,11 +690,11 @@ res = r'''digraph
   }
   {
     node [shape = circle, style = rounded, width = 0.5]
-    0 [label = "0, 0", shape = box]
-    1 [label = "2, 0", shape = box, color = DimGray]
-    2 [label = "0, 1", shape = box]
-    3 [label = "2, 1", shape = box, color = DimGray]
-    4 [label = "1, 2", shape = box]
+    0 [label = "0, (0, !\\e)", shape = box]
+    1 [label = "2, (0, !\\e)", shape = box, color = DimGray]
+    2 [label = "0, (1, \\e)", shape = box]
+    3 [label = "2, (1, \\e)", shape = box, color = DimGray]
+    4 [label = "1, (2, !\\e)", shape = box]
   }
   I0 -> 0
   0 -> 1 [label = "\\e", color = DimGray]
